# Metering Gateway API Specification 
---

## 1 Information
  此文件描述Metering Gateway的兩個API。第一個Subscription Mapping API，是當使用者在訂閱服務時，Service Broker 或 Solution Ready Platform (SRP)需要通知Metering Gateway 使用者所訂閱的服務資訊；第二是Usage Collecting API，當Service Metering Agent量測到的Service使用量資料透需過此API 傳送給Metering Gateway。
### 1.1 URI
  基本的URI 為**http://api-billing-mg{cf-space}.{cf-domain}/{version}** 。 
{cf-space}為Metering Gateway所運行的環境space名稱，develop為-develop，stage為-stage，production則無需填寫； {cf-domain}為Metering Gateway所運行的環境域名；version為API版號，此版版號為v1，詳細各環境URI如下表所示。


| iii-cflab                             ||
|------------|:-----------------------------|
| develop    |  http://api-billing-mg-develop.iii-cflab.com/v1   |
| stage      |  http://api-billing-mg-stage.iii-cflab.com/v1     |
| production |  http://api-billing-mg.iii-cflab.com/v1   |

| Azure HK                             ||
|------------|:-----------------------------|
| develop    | https://api-billing-mg-develop.wise-paas.com/v1 |
| stage      | https://api-billing-mg-stage.wise-paas.com/v1 |
| production | https://api-billing-mg.wise-paas.com/v1 |

| Azure CN                             ||
|------------|:-----------------------------|
| stage      | https://api-billing-mg-stage.wise-paas.com.cn/v1 |
| production | https://api-billing-mg.wise-paas.com.cn/v1 |



## 2 API Functions


### 2.1 Subscription Mapping
  當使用者訂閱服務時，所產生的訂閱服務資訊，需要由Service Broker 或SRP使用create方法通知Metering Gateway；而當使用者要取消訂閱服務時，Service Broker 或SRP使用delete方法通知Metering Gateway，使用者取消訂閱服務，兩個API的方法如下表。

| Method |          HTTP Request        |
|--------|:-----------------------------|
| create |  POST /mapper/subscription   |
| delete |  POST /mapper/unsubscription |


**Create**  
  Service Broker 或 SRP使用create方法通知Metering Gateway使用者的訂閱服務資訊。使用基礎URL **`http://api-billing-mg.{cf-domain}/v1`** 加HTTP Request 呼叫該方法。 

- **HTTP Request** 

	`POST /mapper/subscription`
    
- **HTTP Headers** 

	```
	headers: 
	{
	    'Content-Type': 'application/json'
	}
	```

- **Request Body** 
	
    ```
    {
    	 "service_name": "<string>",
    	 "consumer_id": "<string>", 
    	 "organization_id": "<string>",
    	 "space_id": "<string>",
    	 "plan_id": "<string>"
    }
    ```

- **Response Http status code**

    |  Code	 |             Message           |
    |--------|:------------------------------|
    |  200	 |  OK                           |
    |  400	 |  Bad Request                  |
    |  405	 |  Method Not Allowed           |
    |  409	 |  Conflict                     |
    |  415	 |  Unsupported Media Type       |



- **Response body** 

  Response Http status code 為400時，Message會有相關錯誤訊息。

    ```
    {
    	"Message": "<String>"
    }
    ```


**Delete**

- **HTTP Request** 

	`POST /mapper/unsubscription`
    
- **HTTP Headers** 

	```
	headers: 
	{
    	'Content-Type': 'application/json'
	}
	```

- **Request Body** 
	
    ```
    {
    	 "service_name": "<string>",
    	 "consumer_id": "<string>"
    }
    ```


- **Response Http status code**

    |  Code	 |             Message           |
    |--------|:------------------------------|
    |  200	 |  OK                           |
    |  404	 |  Not Found                    |
    |  405	 |  Method Not Allowed           |
    |  415	 |  Unsupported Media Type       |


- **Response body**  

  Response Http status code 為400時，Message會有相關錯誤訊息。

    ```
    {
    	"Message": "<String>"
    }
    ```


### 2.2 Usage Collecting
  此API目的是為了蒐集各種service用量資料，service用量資料由該Service Metering Agent進行使用量的量測，使Batch insert 方法提供Service Metering Agent 將所量測的service用量資料以批量形式傳送給Metering Gateway。

| Method |          HTTP Request        |
|--------|:-----------------------------|
| Batch insert |  POST /collection/usagelist      |

**Batch insert**    
  servic量測用量時，每個量測目標都會有一組對應的consumer_id與量測值，這樣的資料為一筆量測資訊，將多筆量測目標資訊，組成下面Request Body的Json格式傳給Metering Gateway。
- **HTTP Request** 

	`POST /collection/usagelist`
    
- **HTTP Headers** 

	```
	headers: 
	{
	    'Content-Type': 'application/json'
	}
	```

- **Request Body** 
	
    ```
    {
          "usages": [
        {
            "start": <timestamp>,
            "end": <timestamp>,
            "service_name": "<string>",
            "consumer_id": "<string>",
            "measured_usage": [
              {
                "measure": "<string>",
                "quantity": <double>
              } ]
        },…]
    }
    ```


- **Response Http status code**

    |  Code	 |             Message           |
    |--------|:------------------------------|
    |  200	 |  OK                           |
    |  400	 |  Bad request                  |
    |  405	 |  Method Not Allowed           |
    |  415	 |  Unsupported Media Type       |


- **Response body** 

  Response Http status code 為400時，Message會有相關錯誤訊息。

    ```
    {
    	"Message": "<String>"
    }
    ```


## 3 Parameters


|    Property name   |	Value |          Description          |
|--------------------|:-------|:------------------------------|
| service_name	     | string | 使用者所訂閱服務或SRP的名稱，詳細對照如Service與service_id、consumer_id對照表。|
| consumer_id		 | string |	使用者訂閱服務時，Service Broker 建立Service instance時所產生的service_instance_id或SRP所產生的唯一值id。|
| organization_id	 | string |	使用者所訂閱服務或Jenkins安裝SRP所使用的organization ID。|
| space_id	         | string |	使用者所訂閱服務或Jenkins安裝SRP時所使用的space GUID。|
| plan_id	         | string |	使用者訂閱服務時所選擇plan的plan id。|
| start	             | Long   | 量測資料區間的起始時間，使用Unix epoch表示到milliseconds。|
| end	             | Long   | 量測資料區間的結束時間，使用Unix epoch表示到milliseconds。|
| measured_usage	 | list   |	Service Metering Agent的量測值列表。|
| measure		     | string | Service被量測的名稱，(ex., 資料庫服務的storage, transaction)。|
| quantity		     | double | Service被量測的量測值。|


**Service與service_id、consumer_id對照表**

| Service/SRP |	service_name |          consumer_id                   |
|-------------|:-------------|:---------------------------------------|		
| PostgreSQL  |	“postgresql” | PostgreSQL service配給使用者的DB name。|
| MongoDB     |	“mongodb”    | MongoDB service配給使用者的DB name。   |
| IoT Hub     |	“iot-hub”    | IoT Hub service 的vhost。         |
| RMM	      |“rmm”	     | 待填								      |
| SCADA	      |“scada”	     | 待填									  |
| Condor DM   |	“condor-dm”  | 待填									  |



***Reference***
1.	JSON. URL [http://json.org/](http://json.org/)