Cloud Foundry CLI Guide
===========

Cloud Foundry CLI
----

### 1.1 Introduction

This topic describes how to use Cloud Foundry Command Line Interface (cf CLI)

### 1.2 Purpose

Provide a command line tool (cf CLI) for user to interact with Cloud Foundry. So you can use cf CLI to management your services, apps, orgs, and spaces...etc

### 1.3 Usage

`cf [global options] command [arguments...] [command options]:`

| Option	| Description |
|-------    |-----------  |
|-help, -h  |	Show help |
|-v	        |Print API request diagnostics to stdout|

Command types
--------

### 2.1 Getting Started


|Command|Description|
|-------|-----------|
|login	|Log user in|
|help	|Show help|



### 2.2 App lifecycle management


|Command|Description|
|-------|-----------|
|push	|Push a new app or sync changes to an existing app|
|start	|Start an app|
|stop	|Stop an app|
|delete	|Delete an app|
|apps	|List all apps in the target space|
|app	|Display health and status for app|
|ssh	|SSH to an application container instance|



Getting Started
--------

### 3.1 Login

**3.1.1 Name**

login - Log user in

**3.1.2 Usage**


`cf login [-a API_URL] [-u USERNAME] [-p PASSWORD] [-o ORG] [-s SPACE] --skip-ssl-validation`

|Option name	|Description|
|---------------|-----------|
|-a	|API endpoint	| 
|-u	|Username	|   
|-p	|User password	| 
|-o	|User organization	| 
|-s	|User space	|  
|--skip-ssl-validation	|Skip verification of the API endpoint	| 

**3.1.3 Example**

`cf login --skip-ssl-validation -a https://api.iii-cflab.com -u tic100 -p passw0rd -o tic100 -s team1`


### 3.2 Help

**3.1.1 Name**

help - Show help

**3.1.2 Usage**


`cf help [COMMAND]`

|Option name |Description|
|------------|-----------|
|-a	|All available CLI commands| 

**3.1.3 Example**


`cf help -a`



App lifecycle management
------

### 4.1 Push

**4.1.1 Name**

push - Push a new app or sync changes to an existing app

**4.1.2 Usage**

`
cf push APP_NAME [-p PATH] [-b BUILDPACK_NAME] [-i NUM_INSTANCES] [-k DISK] [-m MEMORY]`




|Option name	|Description	|
|--------------|-----------|
|-p	|Path to app directory or to a zip file of the contents of the app directory| 
|-b	|Custom buildpack by name (e.g. my-buildpack) or Git URL (e.g. https://github.com/cloudfoundry/java-buildpack.git) or Git URL with a branch or tag (e.g. https://github.com/cloudfoundry/java-buildpack.git#v3.3.0 for v3.3.0 tag)	| 
|-i	|Number of app instances	|
|-m	|App instance memory limit (e.g. 256M, 1024M, 1G)	| 
|-k	|App instance disk limit (e.g. 256M, 1024M, 1G)	 | 

**4.1.3 Example**


`cf push example_app -p test.zip -b java_buildpack_offline -i 1 -m 1G -k 1G`



### 4.2 Start

**4.2.1 Name**

start - Start an app

**4.2.2 Usage**


`cf start APP_NAME`

**4.2.3 Example**


`cf start example_app`

### 4.3 Stop

**4.3.1 Name**

stop - Stop an app

**4.3.2 Usage**


`cf stop APP_NAME`

**4.3.3 Example**


`cf stop example_app`



### 4.4 Delete

**4.4.1 Name**

delete - Delete an app

**4.4.2 Usage**


`cf delete APP_NAME [-r] [-f]`


|Option name|Description|
|--------------|----------------|
|-r	|Also delete any mapped routes| 	
|-f	|Force deletion without confirmation| 

**4.4.3 Example**


`cf delete example_app -f`



### 4.5 Apps

**4.5.1 Name**

apps - List all apps in the target space

**4.5.2 Usage**


`cf apps`

**4.5.3 Example**


`cf apps`




### 4.6 App

**4.6.1 Name**

app - Display health and status for app

**4.6.2 Usage**


`cf app APP_NAME`

**4.6.3 Example**


`cf app example_app`



### 4.7 Ssh

**4.7.1 Name**

ssh - SSH to an application container instance

**4.7.2 Usage**


`cf ssh APP_NAME`

**4.7.3 Example**


`cf ssh example_app`



Reference
--------

- Cloud Foundry CLI Reference Guide. URL:https://docs.cloudfoundry.org/cf-cli/cf-help.html
