EI-Connect
=====

**EI-Connect – a set of libraries for communicate between device and IoT/WISE-PaaS Server.**
Advantech provides a set of libraries for communicate and exchange information between a device and IoT/WISE-PaaS Server, called EI-Connect Libraries.
The EI-Connect Libraries provides a rich set of user-friendly, intelligent and integrated interfaces, which speeds development, enhances security and makes agent application easier and simpler to communicate with IoT/WISE-PaaS Server.
The framework has several benefits:

* Standardization - The communication protocol is based on the [MQTT protocol](http://mqtt.org/) to communicate and exchange data with WISE-PaaS Server. The IoT sensor data report format is following the [IPSO Spec.](http://www.ipso-alliance.org/) in [JSON format](http://www.json.org/).
* Portability - Whole framework is written in C language and follow the [ANSI C Standard](https://en.wikipedia.org/wiki/ANSI_C), that C compilers are available for most systems and are often the first compiler provided for a new system.


Architecture
-----

The EI-Connect Libraries is designed for several purpose:

* Lightweight: The WISECore is focused on the handshake protocol to communicate with WISE-PaaS Server.
    * User can implement a lightweight application to communicate with WISE-PaaS Server easily.
* Various Connectivity Protocol: Extract the Connectivity Part as a WISEConnector Module.
    * In WISEConnector we can implement various connectivity protocol, such as: MQTT, AMPQ, Web Socket, HTTP, to migrate to different connectivity protocol easily.
    * For now, we support the MQTT connectivity protocol.
  
![](../uploads/images/EI-Connect/ei-connect-01.png)

**Lightweight**

To porting a full function agent application to a resource restricted platform is difficult. So we are trying to extract the core functions from past experience to a single library called WISE Core. And we can port this small library into resource restricted platform easier.

![](../uploads/images/EI-Connect/ei-connect-02.png)

**Various Connectivity Protocol**

To support various connectivity protocols, we define the common connectivity APIs for EI-Connect, we called WISE Connector APIs. We can easily connect our EI-Connect to various Cloud Server by changing Connector Module at compile time, If those Connector Module is implemented with WISE Connector APIs. To support Microsoft Azure IoTHub, we need to implement a WISE Connector that integrate the Azure IoT Suite.

![](../uploads/images/EI-Connect/ei-connect-03.png)

EI-Connect Handshake Protocol
------

Define the MQTT publish/subscribe topics and payload data JSON format.
Reference to page "[Handshake Protocol](./handshake-protocol.md)". The topic and message format is defined in "[WISE-PaaS header file](https://github.com/ADVANTECH-Corp/EI-Connect/blob/master/Include/wise/wisepaas_02_def.h)".

Library APIs
-------

* **WISECore API** - The document and sample code of WISECore APIs to describe how to use it. The demonstrate sample code is called "WISECoreSample". Reference to page "[WISECore API](./wisecore-api.md)"
* **WISEConnector API** - The document to describe the API definitations. Reference to page "[WISEConnector API](./wiseconnector-api.md)"

Reference
------

* [EI-Connect Handshake Protocol](./handshake-protocol.md)
* [WISECore API](./wisecore-api.md)
* [WISEConnector API](./wiseconnector-api.md)

Source Code
------

* [*EI-Connect*](http://advgitlab.eastasia.cloudapp.azure.com/EI-Connect/EI-Connect.git)

