WISEConnector API
===

We define the Connection API let user to implement custom MQTT library into EI-Connect to support various target platform, such as X86, arm based risc,... etc. We provide two library: "MQTTConnector" and "MosquittoCarrier" to demonstrate how to integrate the Mosquitto MQTT library into EI-Connect. In future, we may implement the WISEConnector API to support or migrate to differnet kind communication protocl, such as AMQP.

WISEConnector APIs
---

**List**

```cpp
bool wc_initialize(char const * devid, void* userdata);
```

Description

- Initialize the client instance.

Parameters

- devid[POINTER]
	- String to use as the client id. If NULL, a random client id will be generated.
- userdata[POINTER]
	- the unstructured pointer to reference to user data.

Return Values

- true/false

```cpp
bool wc_initialize(char const * devid, void* userdata);
```

Description

- Call to free resources associated with the library.

Parameters

- None

Return Values

- None

```cpp
void wc_callback_set( WC_CONNECT_CALLBACK connect_cb, WC_LOSTCONNECT_CALLBACK lostconnect_cb, WC_DISCONNECT_CALLBACK disconnect_cb, WC_MESSAGE_CALLBACK message_cb);
```

Description

- Set the connect callback. This is called when the server response to a connection.

Parameters

- connect_cb[POINTER]
	- a callback function in the following form: void callback(void *userdata)
- lostconnect_cb[POINTER]
	- a callback function in the following form: void callback(void *userdata)
- disconnect_cb[POINTER]
	- a callback function in the following form: void callback(void *userdata)
- message_cb[POINTER]
	- a callback function in the following form: void callback(const char* topic, const void* payload, const int payloadlen, void *userdata)

Return Values

- None

```cpp
bool wc_tls_set(const char *cafile, const char *capath, const char *certfile, const char *keyfile, const char* password);
```

Description

- Configure the client for certificate based SSL/TLS support. Must be called before <wc_connect>.

Parameters

- cafile[POINTER]
	- path to a file containing the PEM encoded trusted CA certificate files. Either cafile or capath must not be NULL.
- capath[POINTER]
	- path to a directory containing the PEM encoded trusted CA certificate files. See mosquitto.conf for more details on configuring this directory. Either cafile or capath must not be NULL.
- certfile[POINTER]
	- path to a file containing the PEM encoded certificate file for this client. If NULL, keyfile must also be NULL and no client certificate will be used.
- keyfile[POINTER]
	- path to a file containing the PEM encoded private key for this client. If NULL, certfile must also be NULL and no client certificate will be used.
- password[POINTER]
	- if keyfile is encrypted, set the password to allow your client to pass the correct password for decryption.

Return Values

- true/false

```cpp
bool wc_tls_psk_set(const char *psk, const char *identity, const char *ciphers);
```

Description

- Configure the client for pre-shared-key based TLS support. Must be called before <wc_connect>.

Parameters

- psk[POINTER]
	- the pre-shared-key in hex format with no leading "0x".
- identity[POINTER]
	- the identity of this client. May be used as the username depending on the server settings.
- ciphers[POINTER]
	- a string describing the PSK ciphers available for use. See the "openssl ciphers" tool for more information. If NULL, the default ciphers will be used.

Return Values

- true/false.

```cpp
bool wc_connect(char const * ip, int port, char const * username, char const * password, int keepalive, char* willtopic, const void *willmsg, int msglen );
```

Description

- Connect to server.

Parameters

- ip[POINTER]
	- the hostname or ip address of the server to connect to.
- port[NUMBER]
	- the network port to connect to. Usually 1883.
- username[POINTER]
	- the username to send as a string, or NULL to disable authentication.
- password[POINTER]
	- the password to send as a string. Set to NULL when username is valid in order to send just a username.
- keepalive[NUMBER]
	- the number of seconds to kept connection.
- willtopic[POINTER]
	- the topic on which to publish the will.
- willmsg[POINTER]
	- pointer to the data to send. If payloadlen > 0 this must be a valid memory location.
- msglen[NUMBER]
	- the size of the willmsg (bytes).

Return Values
- true/false

```cpp
bool wc_disconnect(bool bForce);
``` 

Description

- Disconnect from the server.

Parameters
- bForce[BOOLEAN]
	- floag to dsiconnect without waiting disconnect packet send.

Return Values
- true/false

```cpp
bool wc_publish(const char* topic, const void *msg, int msglen, int retain, int qos);
```

Description

- Publish a message on a given topic.

Parameters

- topic[POINTER]
	- the topic to publish message. 
- msg[POINTER]
	- pointer to the data to send. If msglen > 0 this must be a valid memory location.
- msglen[NUMBER]
	- the size of the payload (bytes). 
- retain[NUMBER]
	- enable flag to retain this message in broker. 
		- 1:means enable
		- 0: means disable
- qos[NUMBER]
	- QoS level:
		- 0: The broker/client will deliver the message once, with no confirmation.
		- 1: The broker/client will deliver the message at least once, with confirmation required.
		- 2: The broker/client will deliver the message exactly once by using a four step handshake. 

Return Values

- true/false

```cpp
bool wc_subscribe(const char* topic, int qos);
```

Description

- Subscribe to a topic.

Parameters

- topic[POINTER]
	- the topic to publish message. 
- qos[NUMBER]
	- QoS level:
		- 0: The broker/client will deliver the message once, with no confirmation.
		- 1: The broker/client will deliver the message at least once, with confirmation required.
		- 2: The broker/client will deliver the message exactly once by using a four step handshake. 

Return Values

- true/false

```cpp
bool wc_unsubscribe(const char* topic);
```

Description

- Unsubscribe from a topic.

Parameters

- topic[POINTER]
	- the topic to publish message. 

Return Values

- true/false

```cpp
bool wc_address_get(char *address);
```

Description

- To get local IP address.
 
Parameters

- address[POINTER]
	- to get local IP address.

Return Values

- true/false

```cpp
const char* wc_current_error_string_get();
```

Description

- Call to obtain a const string description the error number.

Parameters

- rc[NUMBER]
	- the error number.

Return Values

	- string pointer.

**Callback Function**

```cpp
typedef void (*WC_MESSAGE_CALLBACK)(const char* topic, const void* payload, const int payloadlen, void *userdata);
```

Description

- define the message receive callback function.

Parameters

- topic[POINTER]
	- MQTT topic string
- payload[POINTER]
	- the packet string pointer.
- payloadlen[NUMBER]
	- received packet length.
- userdata[POINTER]
	- unformated pointer to carry the user defined structure.

Return Values

- None.

```cpp
typedef void (*WC_CONNECT_CALLBACK)(void *userdata);
```

Description
- define the on connected callback function.

Parameters

- userdata[POINTER]
	- unformated pointer to carry the user defined structure.

Return Values

- None.

```cpp
typedef void (*WC_LOSTCONNECT_CALLBACK)(void *userdata);
```

Description

- define the on lost connect callback function.

Parameters

- userdata[POINTER]
	- unformated pointer to carry the user defined structure.

Return Values

- None.

```cpp
typedef void (*WC_DISCONNECT_CALLBACK)(void *userdata);
```

Description

- define the on disconnect callback function.

Parameters

- userdata[POINTER]
	- unformated pointer to carry the user defined structure.

Return Values

- None.
