RMM Agent
===

**RMM Agent – a software development framework to communicate between device and RMM Server.**

Advantech provides a software development framework to communicate and exchange information between a device and RMM Server, called a RMM Agent framework.

The RMM Agent framework is based on [EI-Agent](documents/EI-Agent/ei-agent.md) framework and provides a rich set of user-friendly, intelligent and integrated interfaces, which speeds development, enhances security and makes agent application easier and simpler to communicate with RMM Server.

The framework has three benefits:

- Standardization - The [EI-Agent](documents/EI-Agent/ei-agent.md) communication protocol is based on the [MQTT protocol](http://mqtt.org/) to communicate and exchange data with RMM Server. The IoT sensor data report format is following the [IPSO Spec](https://www.ipso-alliance.org/). in [JSON format](http://www.json.org/).
- Portability - Whole framework is written in C language and follow the [ANSI C Standard](https://en.wikipedia.org/wiki/ANSI_C), that C compilers are available for most systems and are often the first compiler provided for a new system.
- Scalability - The RMM Agent Framework is functional partitioning into discrete scalable, reusable modules, and plug & playable.

For scalability, Advantech implements several **Software Modules** to access sensor data or control target device, we called **Plugins** (or Handlers we called before).

Each plugin can be designed to handle specific jobs, such as:

- **Sensor Plugin**: the plugin access sensor data through **Sensor driver** or **3rd party library**.
- **Remote Control Plugin**:  the plugin execute remote command on target device.

Based on those Plugins, Advantech also defined a set of APIs, called **Plugin (Handler) APIs**, to support custom plugins implementation.

User can implement their own Handler with these Plugin (Handler) APIs to access their sensor data or control their devices and plugged by RMM Agent to communicate with RMM Server.

Architecture
---

In **RMM Agent Framework**, it includes two parts: **Agent Application** and **Plugins**.

- In **Provisioning & Communication** block, There are defined two structures:
	1. **Platform Profiler** structure describe the target platform, include: OS version, SN, Device Name, MAC address, …
	2. **Configuration** structure describe how to connect to RMM server, such as Server IP, Listen Port, TLS/SSL setting, …
	
 And the **Agent Client** is the main library of RMM Agent to communicate with RMM Server, based on EI-Agent with MQTT protocol.


- In **Core Management** block,
	1. The **Core Manager** integrates and manages the resources and keeps alive.
	2. The **Plugin Loader** is responsible to load and manage the plugins, that indicated in module_config.xml file.
	3. The **RMM Command** is responsible to handle the commands that interact with Agent, such as: Agent Rename or Agent Update.
 

- The **Plugins** part includes the plugins we implement for RMM Agent, such as SUSI Control, Power On/Off. And any other custom plugins that implemented follow our Handler APIs.

<img src="../uploads/images/RMM/rmmagent01.png" width="600">

Sample Application
---

- **[Sample Client](sample-client.md)** - Simple client application example.


RMM Plugins
---

In RMM Agent, Advantech provide several Plugins (Handlers) and can be classified in three different categories:

- Monitoring Plugins - sensor data monitoring and control.

> Such as: Hard-disk Monitor, SUSI Control, Network Monitor, Process Monitor, IoT Gateway and Modbus.

- Control Plugins - remote control target device.

> Such as: Power On/Off, Remote KVM, Screenshot, Terminal, Protection and Recovery.

- Agent Improvement Plugins - to increase the agent reliability or functionality.

> Such as: Server Redundancy.

![](../uploads/images/RMM/rmmagent02.png)

Sample Plugin
---
- **[Sample Plugin](sample-plugin.md)** - Simple plugin example.  

- **Modbus Handler** - version 2.0 only support Modbus-TCP, Modbus-RTU supported since version 3.0.

**Modbus handler** is one of plugins in RMM Agent framework and is used to collect devices’ data based on Modbus TCP/RTU protocol.
The library used in Modbus handler is [libmodbus](http://libmodbus.org/documentation/), which is a 3rd party free library to send/receive data according to the Modbus protocol.
This library is written in C and supports RTU (serial) and TCP (Ethernet) communication. The license of libmodbus is LGPL v2.1.

The flow of RMM Agent communicating with a device based on Modbus by Modbus handler is described as follows.

The RMM Agent uses the Modbus handler as the interface to handle all the communication with the device based on Modbus protocol.

- Steps of Modbus Handler:
	- **Step1** Modbus handler reads in INI file and is ready to set up connection with device according to the configuration in the INI file.
	- **Step2** Modbus handler establishes a Modbus connection
	- **Step3** RMM Agent communicates with the device.
	- **Step4** If you no more need the communication, RMM Agent can disconnect with the device.

<img src="../uploads/images/RMM/rmmagent03.png" width="600">

- **GPS Handler** - version 1.0 only support [NMEA 0183](https://en.wikipedia.org/wiki/NMEA_0183) format with serial port.

**GPS Handler** is used to parse the GPS stream in NMEA 0183 format and report current position to server continuously.The report information include: Serial Port Name, Mode, Timestamp, Latitude, Longitude and Altitude.

- Mode 2 means the GPS device only support two-dimensional information without Altitude.
- Mode 3 means the GPS support three-dimensional information with Altitude.

<img src="../uploads/images/RMM/rmmagent04.png">

The GPS Handler config file is only need to define the serial port attribute, include: Serial Port and Baud Rate.

The default GPSHandler.ini is read stream from com 1, and the baud rate is 9200.

    [Platform]
    SerialPortNum=1
    Baud=9200

- **RFID Handler** - version 1.0 only support [FAVITE](http://www.favite.com/index.asp) low level command protocol.

**RFID Handler** is interacted with FAVITE RFID Reader with low level command protocol through serial port.

The report information include: Serial Port, Baud Rate, Parity Mode, Stop Bits, Byte Size, Manufacturer Name, Product Name, Serial Number and EPC Code.

![](../uploads/images/RMM/rmmagent05.png)

The RFID Handler config file is only need to define the serial port attribute, include: Serial Port and Baud Rate. The default RFIDHandler.ini define the serial port with read stream form com1, baud rate is 115200, stop bits is 1, parity mode is none and byte size is 8. And enable the debounce with set the toggle target to disable and query command inventory session flag to 1.

    [Platform]
    SerialPortNum=1
    Baud=115200
    StopBits=1
    Parity=0
    ByteSize=8
    # Toggle 0:disable, 1:enable.
    # Inventory Session 0:s0, 1:s1, 2:s2, 3:s3
    # Set toggle:0 with Session:0~4 in different trigger rule to support debounce.
    Toggle=0
    Session=1


Source Code
---

* [*https://github.com/ADVANTECH-Corp/EI-Agent*](https://github.com/ADVANTECH-Corp/EI-Agent)

Document
---

