SSO User Guide
=========

What is WISE-PaaS SSO?
-------------

The WISE-PaaS SSO is a Single [Sign-On System](https://en.wikipedia.org/wiki/Single_sign-on) which make developers add account mechanism to their applications easily.

WISE-PaaS SSO Integration Guide
----------------


**Token explanation**

• WISEAccessToken

1. Expired in one hour
2. Checke place: Cookies, attributes: HttpOnly(backend), secure

• WISEName

1. Expired in one hour
2. Checke place: Cookies, attributes: secure
3. Used by Frontend

• WISEAppToken

1. Expired in one hour
2. Header (Authorization: Bearer )
3. Used by native app

• RefreshToken

1. Expired in five day
2. Use to get new WISEAccessToken or WISEAppToken



**Frontend Sign in**

![](../uploads/images/SSO/frontendSignIn.png)


**Native Sign in**

![](../uploads/images/SSO/nativeSignIn.png)

**Tic 100 role of a group permission control**

![](../uploads/images/SSO/roleOfTic100.png)

**SSO scenario for SaaS backend User access SaaS but has not signed in**

![](../uploads/images/SSO/scenarioBackend.png)

1. An WISE-PaaS User access a SaaS which has been deployed on WISE-PaaS.
2. (1)SaaS doesn’t find JSON Web Token (JWT) token stored in cookies. 
    (2)SaaS redirect web page to Auth Service to verify identity of this user
3. Auth Service redirect to Azure AD login page to let User login.
4. After login success, Azure AD will response an authorization code to Auth Service.
5. Auth Service use authorization code to get access token / refresh token from Microsoft Graph. Then Auth Service repack tokens to WISEAccessTokens and response to SaaS and store
6. SaaS let User use it!
7. If User want to access Other SaaSs, they don’t need to sign in again!


**SSO scenario for SaaS backend- User access SaaS and has already signed in**

![](../uploads/images/SSO/scenarioBackendSigned.png)

1. An WISE-PaaS User access a SaaS which has been deployed on WISE-PaaS.
2. SaaS find JSON Web Token (JWT) stored in cookies 
    SaaS access Auth Service api to verify token during a period(frequency need to discuss).
3. Auth Service return token verify result. 

    (1) status: ok(200) 

    (2) status: Bad request(400) jwt expire need to refresh for user (reference tokenExpireFlow.png) 
![](../uploads/images/SSO/tokenExpireFlow.png)

    (3) status: Unauthorized(401) The token provided not valid (maybe show error and redirect sign in page)

4. SaaS let User use it!
5. If User want to access Other SaaSs, they don’t need to sign in again!


**SSO scenario for SaaS Native app - Access SaaS**

![](../uploads/images/SSO/scenarioNativeApp.png)

1. An WISE-PaaS Native app access a SaaS which has been deployed on WISE-PaaS.
2. (1)SaaS find JSON Web Token (JWT) stored in header (Authorization Bearer ). 
    (2)SaaS access Auth Service api to verify token during a period(frequency need to discuss).
3. Auth Service return token verify result. 
    (1) status: ok(200) 
    (2) status: Bad request(400) jwt expire need to refresh for user 
    (3) status: Unauthorized(401) The token provided not valid (show error) 
4. SaaS let App use it!
5. If APP want to access Other SaaSs, they don’t need to request WISEAppToken until it expired!



**API document**

http://wise-paas-documentation.docs.wise-paas.com/Documentation/#!apidoc/SSO/api.md

WISE-PaaS SSO Example
--------

**Purpose**

Integrate a custom web application with WISE-PaaS SSO.

**Procedure**

HTML code

    <!DOCTYPE html>
    <html>
    
    <head>
    <meta charset="UTF-8">
    <title>WISE-PaaS SSO System</title>
    <link rel="stylesheet" type="text/css" href="bower_components/bootswatch/cosmo/bootstrap.min.css">
    <link rel="stylesheet" href="styles/index.css">
    </head>
    
    <body>
    <div class="wrapper">
    <div class="container">
    <img class="img-responsive" src="images/logo.png" style="margin: 0 auto;">
    <form class="form" onsubmit="return false;">
    <div class="panel panel-default" style="border-color: #000000; background-color: rgba(255, 255, 255, 0.5);">
    <div class="panel-body">
    <h4 class="text-center" id="wiseName" style="display: none; color: #000000"></h4>
    <button class="btn btn-primary btn-block" id="signInBtn">Sign in with Azure AD</button>
    <button class="btn btn-success btn-block" id="signUpBtn">Sign up to Azure AD</button>
    <button class="btn btn-primary btn-block" id="signOutBtn" style="display: none;">Sign out</button>
    <h4 class="text-right" id="resetPassword"><a href="resetPassword.html">Reset password</a></h4>
    <h4 class="text-right" id="changePassword" style="display: none;"><a href="https://go.microsoft.com/fwlink/?LinkID=522342">Change password</a></h4>
    </div>
    </div>
    </form>
    </div>
    <ul class="bg-bubbles">
    <li></li>
    <li></li>
    <li></li>
    <li></li>
    <li></li>
    <li></li>
    <li></li>
    <li></li>
    <li></li>
    <li></li>
    </ul>
    </div>
    <script type="text/javascript" src="bower_components/jquery/dist/jquery.min.js"></script>
    <script type="text/javascript" src="bower_components/js-cookie/src/js.cookie.js"></script>
    <script type="text/javascript" src="bower_components/bootstrap/dist/js/bootstrap.min.js"></script>
    <script type="text/javascript" src="scripts/config.js"></script>
    <script type="text/javascript" src="scripts/index.js"></script>
    </body>
    
    </html>


JavaScript (JQuery)

    $(function() {
    var wiseName = Cookies.get('WISEName');
    if (typeof(wiseName) !== 'undefined') {
    $('#wiseName').text('Hello, ' + wiseName + '. Welcome to WISE-PaaS!');
    $('#signInBtn').hide();
    $('#signUpBtn').hide();
    $('#resetPassword').hide();
    $('#wiseName').show();
    $('#signOutBtn').show();
    $('#changePassword').show();
    }
    
    $('#signInBtn').click(function() {
    var redirectUrl = 'https://login.microsoftonline.com/c53d73cb-64c4-4c1d-b972-7f92d1330c39/oauth2/authorize?response_type=code&redirect_uri=' + GLOBAL_CONFIG.hostUrl + '/web/redirectPage.html&client_id=09ea49b3-09fc-4b9a-b452-52563e9d4add&state=' + GLOBAL_CONFIG.hostUrl + '/web/index.html';
    window.location.href = redirectUrl;
    });
    
    $('#signUpBtn').click(function() {
    var redirectUrl = GLOBAL_CONFIG.hostUrl + '/web/signUp.html';
    window.location.href = redirectUrl;
    });
    
    $('#signOutBtn').click(function() {
    var redirectUrl = GLOBAL_CONFIG.hostUrl + '/web/signOut.html';
    window.location.href = redirectUrl;
    });
    });
    
    ! function(a) {
    function b(b, d) {
    function e() {
    if (w) {
    $canvas = a('<canvas class="pg-canvas"></canvas>'), v.prepend($canvas), p = $canvas[0], q = p.getContext("2d"), f();
    for (var b = Math.round(p.width * p.height / d.density), c = 0; b > c; c++) {
    var e = new l;
    e.setStackPos(c), x.push(e)
    }
    a(window).on("resize", function() { h() }), a(document).on("mousemove", function(a) { y = a.pageX, z = a.pageY }), B && !A && window.addEventListener("deviceorientation", function() { D = Math.min(Math.max(-event.beta, -30), 30), C = Math.min(Math.max(-event.gamma, -30), 30) }, !0), g(), o("onInit")
    }
    }
    
    function f() { p.width = v.width(), p.height = v.height(), q.fillStyle = d.dotColor, q.strokeStyle = d.lineColor, q.lineWidth = d.lineWidth }
    
    function g() {
    if (w) {
    s = a(window).width(), t = a(window).height(), q.clearRect(0, 0, p.width, p.height);
    for (var b = 0; b < x.length; b++) x[b].updatePosition();
    for (var b = 0; b < x.length; b++) x[b].draw();
    E || (r = requestAnimationFrame(g))
    }
    }
    
    function h() {
    for (f(), i = x.length - 1; i >= 0; i--)(x[i].position.x > v.width() || x[i].position.y > v.height()) && x.splice(i, 1);
    var a = Math.round(p.width * p.height / d.density);
    if (a > x.length)
    for (; a > x.length;) {
    var b = new l;
    x.push(b)
    } else a < x.length && x.splice(a);
    for (i = x.length - 1; i >= 0; i--) x[i].setStackPos(i)
    }
    
    function j() { E = !0 }
    
    function k() { E = !1, g() }
    
    function l() {
    switch (this.stackPos, this.active = !0, this.layer = Math.ceil(3 * Math.random()), this.parallaxOffsetX = 0, this.parallaxOffsetY = 0, this.position = { x: Math.ceil(Math.random() * p.width), y: Math.ceil(Math.random() * p.height) }, this.speed = {}, d.directionX) {
    case "left":
    this.speed.x = +(-d.maxSpeedX + Math.random() * d.maxSpeedX - d.minSpeedX).toFixed(2);
    break;
    case "right":
    this.speed.x = +(Math.random() * d.maxSpeedX + d.minSpeedX).toFixed(2);
    break;
    default:
    this.speed.x = +(-d.maxSpeedX / 2 + Math.random() * d.maxSpeedX).toFixed(2), this.speed.x += this.speed.x > 0 ? d.minSpeedX : -d.minSpeedX
    }
    switch (d.directionY) {
    case "up":
    this.speed.y = +(-d.maxSpeedY + Math.random() * d.maxSpeedY - d.minSpeedY).toFixed(2);
    break;
    case "down":
    this.speed.y = +(Math.random() * d.maxSpeedY + d.minSpeedY).toFixed(2);
    break;
    default:
    this.speed.y = +(-d.maxSpeedY / 2 + Math.random() * d.maxSpeedY).toFixed(2), this.speed.x += this.speed.y > 0 ? d.minSpeedY : -d.minSpeedY
    }
    }
    
    function m(a, b) {
    return b ? void(d[a] = b) : d[a]
    }
    
    function n() { v.find(".pg-canvas").remove(), o("onDestroy"), v.removeData("plugin_" + c) }
    
    function o(a) { void 0 !== d[a] && d[a].call(u) }
    var p, q, r, s, t, u = b,
    v = a(b),
    w = !!document.createElement("canvas").getContext,
    x = [],
    y = 0,
    z = 0,
    A = !navigator.userAgent.match(/(iPhone|iPod|iPad|Android|BlackBerry|BB10|mobi|tablet|opera mini|nexus 7)/i),
    B = !!window.DeviceOrientationEvent,
    C = 0,
    D = 0,
    E = !1;
    return d = a.extend({}, a.fn[c].defaults, d), l.prototype.draw = function() {
    q.beginPath(), q.arc(this.position.x + this.parallaxOffsetX, this.position.y + this.parallaxOffsetY, d.particleRadius / 2, 0, 2 * Math.PI, !0), q.closePath(), q.fill(), q.beginPath();
    for (var a = x.length - 1; a > this.stackPos; a--) {
    var b = x[a],
    c = this.position.x - b.position.x,
    e = this.position.y - b.position.y,
    f = Math.sqrt(c * c + e * e).toFixed(2);
    f < d.proximity && (q.moveTo(this.position.x + this.parallaxOffsetX, this.position.y + this.parallaxOffsetY), d.curvedLines ? q.quadraticCurveTo(Math.max(b.position.x, b.position.x), Math.min(b.position.y, b.position.y), b.position.x + b.parallaxOffsetX, b.position.y + b.parallaxOffsetY) : q.lineTo(b.position.x + b.parallaxOffsetX, b.position.y + b.parallaxOffsetY))
    }
    q.stroke(), q.closePath()
    }, l.prototype.updatePosition = function() {
    if (d.parallax) {
    if (B && !A) {
    var a = (s - 0) / 60;
    pointerX = (C - -30) * a + 0;
    var b = (t - 0) / 60;
    pointerY = (D - -30) * b + 0
    } else pointerX = y, pointerY = z;
    this.parallaxTargX = (pointerX - s / 2) / (d.parallaxMultiplier * this.layer), this.parallaxOffsetX += (this.parallaxTargX - this.parallaxOffsetX) / 10, this.parallaxTargY = (pointerY - t / 2) / (d.parallaxMultiplier * this.layer), this.parallaxOffsetY += (this.parallaxTargY - this.parallaxOffsetY) / 10
    }
    switch (d.directionX) {
    case "left":
    this.position.x + this.speed.x + this.parallaxOffsetX < 0 && (this.position.x = v.width() - this.parallaxOffsetX);
    break;
    case "right":
    this.position.x + this.speed.x + this.parallaxOffsetX > v.width() && (this.position.x = 0 - this.parallaxOffsetX);
    break;
    default:
    (this.position.x + this.speed.x + this.parallaxOffsetX > v.width() || this.position.x + this.speed.x + this.parallaxOffsetX < 0) && (this.speed.x = -this.speed.x)
    }
    switch (d.directionY) {
    case "up":
    this.position.y + this.speed.y + this.parallaxOffsetY < 0 && (this.position.y = v.height() - this.parallaxOffsetY);
    break;
    case "down":
    this.position.y + this.speed.y + this.parallaxOffsetY > v.height() && (this.position.y = 0 - this.parallaxOffsetY);
    break;
    default:
    (this.position.y + this.speed.y + this.parallaxOffsetY > v.height() || this.position.y + this.speed.y + this.parallaxOffsetY < 0) && (this.speed.y = -this.speed.y)
    }
    this.position.x += this.speed.x, this.position.y += this.speed.y
    }, l.prototype.setStackPos = function(a) { this.stackPos = a }, e(), { option: m, destroy: n, start: k, pause: j }
    }
    var c = "particleground";
    a.fn[c] = function(d) {
    if ("string" == typeof arguments[0]) {
    var e, f = arguments[0],
    g = Array.prototype.slice.call(arguments, 1);
    return this.each(function() { a.data(this, "plugin_" + c) && "function" == typeof a.data(this, "plugin_" + c)[f] && (e = a.data(this, "plugin_" + c)[f].apply(this, g)) }), void 0 !== e ? e : this
    }
    return "object" != typeof d && d ? void 0 : this.each(function() { a.data(this, "plugin_" + c) || a.data(this, "plugin_" + c, new b(this, d)) })
    }, a.fn[c].defaults = { minSpeedX: .1, maxSpeedX: .7, minSpeedY: .1, maxSpeedY: .7, directionX: "center", directionY: "center", density: 1e4, dotColor: "#666666", lineColor: "#666666", particleRadius: 7, lineWidth: 1, curvedLines: !1, proximity: 100, parallax: !0, parallaxMultiplier: 5, onInit: function() {}, onDestroy: function() {} }
    }(jQuery),
    /**
     * requestAnimationFrame polyfill by Erik Möller. fixes from Paul Irish and Tino Zijdel
     * @see: http://paulirish.com/2011/requestanimationframe-for-smart-animating/
     * @see: http://my.opera.com/emoller/blog/2011/12/20/requestanimationframe-for-smart-er-animating
     * @license: MIT license
     */
    function() {
    for (var a = 0, b = ["ms", "moz", "webkit", "o"], c = 0; c < b.length && !window.requestAnimationFrame; ++c) window.requestAnimationFrame = window[b[c] + "RequestAnimationFrame"], window.cancelAnimationFrame = window[b[c] + "CancelAnimationFrame"] || window[b[c] + "CancelRequestAnimationFrame"];
    window.requestAnimationFrame || (window.requestAnimationFrame = function(b) {
    var c = (new Date).getTime(),
    d = Math.max(0, 16 - (c - a)),
    e = window.setTimeout(function() { b(c + d) }, d);
    return a = c + d, e
    }), window.cancelAnimationFrame || (window.cancelAnimationFrame = function(a) { clearTimeout(a) })
    }();
    
    $(function() {
    
    $('#particles').particleground({
    minSpeedX: 0.1,
    maxSpeedX: 0.7,
    minSpeedY: 0.1,
    maxSpeedY: 0.7,
    directionX: 'center', // 'center', 'left' or 'right'. 'center' = dots bounce off edges
    directionY: 'center', // 'center', 'up' or 'down'. 'center' = dots bounce off edges
    density: 10000, // How many particles will be generated: one particle every n pixels
    dotColor: '#eee',
    lineColor: '#eee',
    particleRadius: 7, // Dot size
    lineWidth: 1,
    curvedLines: true,
    proximity: 100, // How close two dots need to be before they join
    parallax: false
    });
    
    });
                                        


**Result**

![](../uploads/images/SSO/wise-paas-sso-example01.png)

WISE-PaaS SSO Release Notes
---------------

**What's New**

Q2 2017 Release

* Sign in
* Sign out
* Sign up
* CRUD for users
* CRUD for apps

