# SQL基本介绍
## 基本知识
    SQL（ Structured Query Language、结构化查询语言）是一种数据库查询语言，用与来与数据库进行交流。
    它已经经明显确立了以自己为标准的关系数据库语言的地位。
    SQL只使用几个简单的词，如select、where、from等。因此，SQL是一个非常易学易用的语言。
    并且，虽然看起来简单，但SQL仍具有强大的能力。（当然，能力仍是有限的，需要与其它高级程序设计语言进行配合使用，才能发挥它的性能。）
    我们可以使用SQL来在关系型数据库中进行查询数据，但SQL还提供另外一些功能。
    
## SQL语言有以下几个部分：
- 	数据定义语言：定义关系模式、删除关系、修改关系模式等。
- 	数据操纵语言：SQL DML提供从数据库查询信息、还有增、删、改等能力（CRUD）。
- 	完整性（SQL DDL)：SQL DDL包括定义完整性约束的命令。
- 	视图定义：DDL\定义视图的命令
- 	事务控制：包括定义事务的开始和结束的命令。
- 	嵌入式SQL和动态SQL：嵌入到通用编程语言，如C、C++、Java等。
- 	授权：SQL DDL包括定义对关系和视图的访问权限的命令


## SQL标准和SQL扩展
    1986年，ANSI（American National Standards Institute）和ISO发布了SQL标准：SQL-86，
    后续的版本包括SQL-89、SQL-92、SQL:1999、SQL:2003、SQL:2006、SQL:2003等等。
    请参考WIKI百科。https://en.wikipedia.org/wiki/SQL

![](images/SQL_NOSQL_00.jpg)

    需要注意的的是，并不是所有的SQL实现都遵循SQL标准的，具体来说，某些实现并没有完全支持SQL标准，
    并且，SQL实现通常还扩展了SQL标准，形成实现自己的特色。这也是SQL在不同的数据库系统之间移植性较差的原因。
    下表同样是来自WIKI百科，它描述了一些SQL扩展。https://en.wikipedia.org/wiki/SQL

![](images/SQL_NOSQL_01.jpg)

    举例来说，不同的SQL实现（供应商）在日期、时间和字符串连接都有不同的处理。
- 1.在Access和SQL Server使用加号+进行连接，而在DB2、PostgreSQL等则使用双竖杆||、MySQL则是使用特殊的函数。
- 2.通配符的使用在不同的扩展、实现中可能用法也不同。通常使用的通配符是百分号%（表示任意字符出现任意多次），而Access则是使用*作为通配符。
- 3.在函数的使用方面，不同的实现也有不同的处理方式。例如，取当前时间。Access使用的是NOW函数;DB2和PostgreDB使用CURRENT_DATE；MySQL和MariaDB使用CURDATE函数……
- 

## ACID性质
ACID是指数据库事务中的四个基本要素（事务可以简单地理解成一组操作）。

其中，A表示原子性Atomicity、C表示一致性 （Consistency）、I表示隔离性（Isolation）、D表示持久性（Durability）。支持事务的数据库系统必须要支持这四个要素。

- 	原子性：一个事务的所有操作要么全部做完，要么全部不做。（如果其中一个操作失败了，需要回滚已经完成的操作。）
- 	一致性：事务的运行不会影响数据库的一致性。在这里，一致性可以理解成某种约束，事务运行前后都需要满足这个约束。
    例如转账前后两个账号的金额总和是不变的：事务运行前金额的总和是多少，事务运行之后也应该是多少。
- 	隔离性：并发的事务之间不会相互影响。
- 	持久性：一旦事务提交，它所做的修改不会因为宕机而丢失，而是会永久性地保存在数据库中。

# NoSQL基本介绍
## 基础知识
    NoSQL起初被称为non-SQL、non-relational，含义为非SQL、非关系。它提供了不同于SQL的数据建模方式。
    SQL的数据建模通常是以“表格”的形式，而NoSQL则支持更为灵活的方式。现在NoSQL通常被理解为Not Only SQL，以表示它有时也提供类似于SQL的查询方式。
    随着WEB 2.0的快速发展，NoSQL数据库系统越来越多地被应用在大数据、Web应用程序中（如社交网站）。
    以上内容参考WIKI： https://en.wikipedia.org/wiki/NoSQL 
    传统的关系型数据库为了要满足事务四个要素ACID，尤其是一致性，这无可避免地带来了一些繁重的操作，使得传统的关系型数据库在性能上有一定的缺陷。
    随着近几年网站的快速发展（如各种各样的论坛、社交网站等）、网站应用数据的大量产生，网站应用对性能、可用性、可扩展性的要求越来越高。
    而这时候，NoSQL凭借着它易扩展、高性能、灵活的数据模型、高可用等优势，开始成为网站应用的首选。
## NoSQL数据库的分类
	常见的NoSQL数据库包括了以下四个分类：
- 1.	key-value存储数据库
- 2.	列存储数据库
- 3.	文档型数据库
- 4.	图形数据库
下表来自百度百科：https://baike.baidu.com/item/NoSQL/8828247?fr=aladdin 
![](images/SQL_NOSQL_02.jpg)

## CAP理论和NoSQL的BASE性质
CAP理论认为，任何分布式数据库只可以具有以下三个性质中的两个：
- 	一致性 consistency
- 	可用性 availability
- 	划分容忍性 partition tolerance
这里的划分的含义是：因为网络等方面的原因，使得多个站点之间无法通信。

相比较于SQL事务的ACID性质，NoSQL要求的BASE特性比较宽松。
毕竟，传统的关系型数据库为了维护数据库的一致性需要大量的额外开销，而对于社交类的网站应用来说，这种短暂的一致性并不会像银行那样造成太严重的后果。
因此，BASE特性并不像ACID性质要求得那么严格。根据CAP理论，有时候为了高可用性，我们需要牺牲一定的一致性。
- 	基本上可用basically available
- 	软状态soft state
- 	最终一致性 eventually consistent：要求解决划分之后，最终所有的副本都一致。可见，BASE特性对一致性的要求没有ACID高。


# SQL和NoSQL（关系型数据库与非关系型数据库）的比较
参考资料：http://www.cnblogs.com/wanghongyun/p/6193912.html 
## 1.	SQL和NoSQL的一个主要差别在于数据存储、建模的方式不同。
关系型数据库是以表的形式来存储的，表的每一行表示一个记录，每一列表示一个属性。不同的表之间有一定的关联，例如有相同的属性列，然后不同的表通过这种关联来一起存储一个事物的属性。并且，可以通过多表连接的方式来读取数据。而对于NoSQL.来说，它并没有表的概念。它的数据被存在在数据集中，例如存储在JSON格式、类似JSON格式。例如，文档型的NoSQL数据库MongoDB将数据存储在BSON中。
另外，SQL中的表需要预定义一个模式，即一个表需要有哪些列/属性，每个属性的存储的数据结构是什么。而对于NoSQL来说，它就没有这个限制。不同的记录之间并不要求有一样的模式。从这个角度上看，NoSQL存储数据的方式比SQL要灵活得多，它可以动态地适应不同的需求。在模式需要改变的时候，SQL改变表的模式会带来很多麻烦，例如底层的数据存储可能需要重新分配空间来适应、有些记录中新增加的属性为空需要设置为Null等。换而言之，非关系型数据库通常更具有扩展性。
## 2.	存储的规范化与存储代价
关系型数据库尽量避免同一份数据存在多份拷贝，以避免更新一份数据时需要更新多份拷贝以保持数据的正确性。这样避免重复的做法可以节约空间上的开销，也有时可以减少在更新数据时的开销。但是作为代价，这种做法会导致一个查询操作可能需要结合多个表才能获取到想要的数据（多表连接）。现在，我们对于存储空间开销的节省并不是那么迫切了。而NoSQL可能存在这样的数据重复，但是通常一个记录可以整块的存储，因此在查询它的时候可以很方便、快速地读取整个数据。
## 3.	结构化查询与非结构化查询 
关系型数据库使用SQL(结构化查询语句)来操作数据。SQL包括了常见的CRUD（增加、查询、更新、删除）。而非关系型数据库则不然。另外，关系型数据库在执行一个查询时常需要连接多张表，而非关系型数据库通常不是这样。
## 4.	ACID与CAP
前文也说过，关系型数据库通常支持事务的ACID四个要素（原子性、持久性、一致性、隔离性），以保持数据的完整性、正确性。而非关系型数据库则支持CAP原理，即在一致性、可用性和分区容忍度三者只可以保持其中的两个。而通常在分布式系统中，分区（例如因网络等原因而引起的站点之间无法通信）是难以避免的，为了在分区发生时，为了保持可用性，只可以牺牲一定的一致性了。非关系型数据库需要满足BASE性质，它只要求最终一致性。


